import ta
import pandas as pd
import numpy as np


def indicators_tech_closePriceOnly(price, date, name):
    """
    Function calculates 31 technical indicators for a price vector

    @param price vector of price
    @param date vector date corresponding to prices
    @param name str name of company/portfolio


    @return df dataframe with price, company/portfolio name and tech indicators
    """
    # convert vector to DataFrame
    df = pd.DataFrame({"price": price})
    df["date"] = date
    df["name"] = name.strip()

    # add columns with indicators
    # Momentum indicators

    PPO = ta.momentum.PercentagePriceOscillator(price)
    df["ppo"] = PPO.ppo()
    df["ppo_hist"] = PPO.ppo_hist()
    df["ppo_signal"] = PPO.ppo_signal()

    # Rate of Change (ROC)
    df["roc"] = ta.momentum.ROCIndicator(price).roc()
    # Relative Strength Index (RSI)
    df["rsi"] = ta.momentum.RSIIndicator(price).rsi()

    # Stochastic RSI
    StockRsi = ta.momentum.StochRSIIndicator(price)
    df["rsi_stoch"] = StockRsi.stochrsi()
    df["rsi_stochrsi_d"] = StockRsi.stochrsi_d()
    df["rsi_stochrsi_k"] = StockRsi.stochrsi_k()

    # True strength index (TSI)
    df["tsi"] = ta.momentum.TSIIndicator(price).tsi()

    # Kaufman’s Adaptive Moving Average (KAMA)
    df["kama"] = ta.momentum.kama(price)

    # Volatility Indicators

    indicator_bb = ta.volatility.BollingerBands(price)
    df["bollinger_mavg"] = indicator_bb.bollinger_mavg()
    df["bollinger_hband"] = indicator_bb.bollinger_hband()
    df["bollinger_lband"] = indicator_bb.bollinger_lband()

    df["bollinger_pband"] = indicator_bb.bollinger_pband()
    df["bollinger_wband"] = indicator_bb.bollinger_wband()

    # Ulcer Index
    df["ulcer"] = ta.volatility.UlcerIndex(price).ulcer_index()

    # Volatility

    # Aroon Indicator
    indocator_aroon = ta.trend.AroonIndicator(price)
    df["aroon_down"] = indocator_aroon.aroon_down()
    df["aroon_ind"] = indocator_aroon.aroon_indicator()
    df["aroon_up"] = indocator_aroon.aroon_up()

    # Detrended Price Oscillator (DPO)
    df["dpo"] = ta.trend.DPOIndicator(price).dpo()

    # EMA - Exponential Moving Average
    df["ema"] = ta.trend.EMAIndicator(price).ema_indicator()

    # KST Oscillator (KST Signal)
    indicator_kst = ta.trend.KSTIndicator(price)
    df["kst"] = indicator_kst.kst()
    df["kst_diff"] = indicator_kst.kst_diff()
    df["kst_sig"] = indicator_kst.kst_sig()

    # MACD
    indicator_macd = ta.trend.MACD(price)
    df["macd"] = indicator_macd.macd()
    df["macd_diff"] = indicator_macd.macd_diff()
    df["macd_signal"] = indicator_macd.macd_signal()

    # SMA Simple MA
    df["sma"] = ta.trend.SMAIndicator(price, window=20).sma_indicator()

    # Schaff Trend Cycle
    df["stc"] = ta.trend.STCIndicator(price).stc()

    # Trix
    df["trix"] = ta.trend.TRIXIndicator(price).trix()

    # WMA Weighted MA
    df["wma"] = ta.trend.WMAIndicator(price).wma()

    # return dataframe
    return df


def french49_generate_indicators_closedPrices(data, companies, date="Date"):
    """
    Function applied in indicators_tech_closePriceOnly function to French dataset. Prices
    in the original dataset are stored using portfolio name, e.g. 'Smoke' = 1,0.5,...

    @param data dataframe with price vector with name E companies.
    @param companies list names of portfolios
    @param date str name of date variable in data

    return df dataframe featuring all
    """

    # init empty dataframe
    df = None

    for symbol in companies:
        if df is None:
            df = indicators_tech_closePriceOnly(data[symbol], data[date], name=symbol)
        else:
            df = df.append(
                indicators_tech_closePriceOnly(data[symbol], data[date], name=symbol)
            )

    # subset data no older then 1970, as only then all indicators feature non-NAN values
    df = (
        df[(df["date"] > "1970-01-01") & (df["date"] <= "2021-11-30")]
        .sort_values(by="date")
        .reset_index()
    )
    df = df.iloc[:, 1:]
    print(int(np.mean(df.price)))

    # check if sum of prices equal number identified during initial setup
    if int(np.mean(df.price)) != 263697:
        raise NameError("Error 001: dataset versions do not match")
        # return None

    # no more NA should be present at this stage

    if df.iloc[1:].isnull().values.any():
        raise NameError("Error 002: Unexpected NA values!")

    return df



import warnings
from tqdm import tqdm
import pandas as pd


def downloader(file=None):
    warnings.filterwarnings("ignore")

    with tqdm(total=100) as pbar:

        # update progress bar
        pbar.update(10)

        data = pd.read_csv(file, compression="zip")

        # update progress bar
        pbar.update(100)

        return data



from refinda.preprocessors.indicators import *
import warnings
from refinda.preprocessors.helper import *
from tqdm import tqdm


def french49(file=None):
    warnings.filterwarnings("ignore")

    with tqdm(total=100) as pbar:

        if file == None:
            data = pd.read_csv(
                "https://mba.tuck.dartmouth.edu/pages/faculty/ken.french/ftp/49_Industry_Portfolios_daily_CSV.zip",
                compression="zip",
                skiprows=[0, 1, 2, 3, 4, 5, 6, 7, 8],
            )
        else:
            data = pd.read_csv(
                file,
                skiprows=[0, 1, 2, 3, 4, 5, 6, 7, 8],
            )

        # update progress bar
        pbar.update(50)

        # exclude last entry indicating coopyrights
        data = data.iloc[0 : data.shape[0] - 1]

        # create date variable
        data["Date"] = data.iloc[:, 0]
        # data['Index'] =  [i for i in range(data.shape[0])]

        # drop unnamed column
        data = data.drop(["Unnamed: 0"], axis=1)

        # select equally weighted
        data = data.iloc[25128 : data.shape[0]]
        data = data.reset_index()

        data = str_to_float(data, date="Date")

        # update progress bar
        pbar.update(10)

        # Restric data to >= 1969-07-01 as this is the first months with data availabiltiy for all 49 portfolios
        data = data.loc[
            (data["Date"] >= "1969-07-01") & (data["Date"] <= "2021-11-30"), :
        ]

        # apply convert_prices function to convert return values to prices using 100 as start
        df = data.iloc[:, 1 : data.shape[1] - 1].apply(convert_prices, axis=0)
        # add date variable again
        df["Date"] = data["Date"]

        # get portfolio names
        portfolios = df.iloc[:, 0 : df.shape[1] - 1].columns

        # calculate indicstors
        df = french49_generate_indicators_closedPrices(df, portfolios)

        # update progress bar
        pbar.update(40)

        return df



from datetime import datetime
import pandas as pd


def str_to_float(data, date="Date"):
    """
    Converts string data float excep Date variable
    """
    for col in data.columns:
        if col != date:
            data[col] = [float(str(x).strip()) for x in data[col].values]
        else:
            data[col] = [
                datetime.strptime(str(x).strip(), "%Y%m%d") for x in data[col].values
            ]
    return data


def convert_prices(dailyReturns, start=100):
    """
    Function converts return values to prices using starting value of start

    @param dailyReturns vector of returns

    @return vector of prices
    """

    df = pd.DataFrame({"dailyReturns": dailyReturns})
    df["close"] = start * (1 + df["dailyReturns"] / 100).cumprod()

    return df["close"]



from setuptools import setup, find_packages

setup(
     name="refinda",
     version="1.0.2",
     description="A Python package for reproducing financial data",
     url="https://github.com/kruthof/refinda",
     author="Garvin Kruthof",
     author_email="",
     license="MIT",
     long_description="file: README.md",
     long_description_content_type="text/markdown",
    packages=find_packages(),
     classifiers=[
         "Programming Language :: Python :: 3",
         "License :: OSI Approved :: MIT License",
         "Operating System :: OS Independent",
         "Intended Audience :: Financial and Insurance Industry",
         "Intended Audience :: Science/Research",
     ],
     python_requires=">=3.7",
     install_requires=[
         "numpy>=1.21.*",
         "pandas>=1.2.*",
         "wrds==3.1.1",
         "tqdm==4.62.3",
         "ta==0.9.0",
     ],
     zip_safe=False,
)


from refinda.datasets.downloader import downloader

"""
Functions that download datasets from the cloud
"""


def simple_pre_processor(data):
    del data["Unnamed: 0"]
    return data


def french_smb_hml_mkt_rf():
    file = "https://dl.dropboxusercontent.com/s/ar3qvysnlveobfx/French_smb_hml_mkt_rf.csv.zip?dl=0"
    data = simple_pre_processor(downloader(file))
    return data


def french_mkt_size_bm_weighted_rf():
    file = "https://dl.dropboxusercontent.com/s/f1u94i2mdp0wznx/French_mkt_size_bm_weighted.csv.zip?dl=0"
    data = simple_pre_processor(downloader(file))
    return data


def french_mkt_size_bm_weighted_smb_hml_rf():
    file = "https://dl.dropboxusercontent.com/s/ohop3y6h1rc7pyn/French_mkt_size_bm_weighted_smb_hml.csv.zip?dl=0"
    data = simple_pre_processor(downloader(file))
    return data


def french_mkt_size_bm_weighted_smb_hml_mom_rf():
    file = "https://dl.dropboxusercontent.com/s/m54ehysgt52zv40/French_mkt_size_bm_weighted_smb_hml_mom.csv.zip?dl=0"
    data = simple_pre_processor(downloader(file))
    return data


def french_10ind_weighted_mkr_rf():
    file = "https://dl.dropboxusercontent.com/s/6mpuerlui6mo264/French_10_mkt_rf_weighted.csv.zip?dl=0"
    data = simple_pre_processor(downloader(file))
    return data
